@@ -133,6 +133,7 @@ ff_open (lua_State *L) {
 	snprintf(s,511,"%s", fontname);
   }
   if (strlen(s)>0) {
+    gww_error_count=0;
 	sf = ReadSplineFont((char *)s,openflags);
 	if (sf==NULL) {
 	  lua_pushfstring(L,"font loading failed for %s\n", fontname);
@@ -255,6 +256,14 @@ static char *make_tag_string (unsigned int field) {
   return (char *)tag_string;
 }
 
+static char featbuf[32] = {0};
+
+static char *make_mactag_string (unsigned int field) {
+  sprintf( featbuf, "<%d,%d>", field>>16, field&0xffff );
+  return (char *)featbuf;
+}
+
+
 static void 
 dump_tag (lua_State *L, char *name, unsigned int field) {
   lua_checkstack(L,2);
@@ -263,6 +272,14 @@ dump_tag (lua_State *L, char *name, unsigned int field) {
   lua_rawset(L,-3);
 }
 
+static void 
+dump_mactag (lua_State *L, char *name, unsigned int field) {
+  lua_checkstack(L,2);
+  lua_pushstring(L,name);
+  lua_pushstring(L,make_mactag_string(field));
+  lua_rawset(L,-3);
+}
+
 #define NESTED_TABLE(a,b,c) {                                           \
     int k = 1;                                                          \
     next = b;															\
@@ -308,7 +325,11 @@ handle_scriptlanglist (lua_State *L, struct scriptlanglist *sll) {
 
 void
 do_handle_featurescriptlanglist (lua_State *L, struct featurescriptlanglist *features) {
-  dump_tag (L,"tag",features->featuretag);
+  if (features->ismac) {
+	dump_mactag (L,"tag",features->featuretag);
+  } else {
+	dump_tag (L,"tag",features->featuretag);
+  }
   lua_newtable(L);
   handle_scriptlanglist(L, features->scripts);
   lua_setfield(L,-2,"scripts");
@@ -658,7 +679,7 @@ void
 handle_splinechar (lua_State *L,struct splinechar *glyph, int hasvmetrics) {
   
   dump_stringfield(L,"name",        glyph->name);
-  dump_intfield(L,"unicodeenc",     glyph->unicodeenc);
+  dump_intfield(L,"unicode",     glyph->unicodeenc);
   lua_createtable(L,4,0);
   lua_pushnumber(L,1);  lua_pushnumber(L,glyph->xmin); lua_rawset(L,-3);
   lua_pushnumber(L,2);  lua_pushnumber(L,glyph->ymin); lua_rawset(L,-3);
