@@ -1,24 +1,22 @@
-/*
-Copyright 1996-2006 Han The Thanh, <thanh@pdftex.org>
+/* pdftoepdf.cc
+   
+   Copyright 1996-2006 Han The Thanh <thanh@pdftex.org>
+   Copyright 2006-2008 Taco Hoekwater <taco@luatex.org>
 
-This file is part of pdfTeX.
+   This file is part of LuaTeX.
 
-pdfTeX is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   LuaTeX is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2 of the License, or (at your
+   option) any later version.
 
-pdfTeX is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   LuaTeX is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
 
-You should have received a copy of the GNU General Public License
-along with pdfTeX; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-$Id: pdftoepdf.cc,v 1.9 2006/09/01 18:06:52 hahe Exp hahe $
-*/
+   You should have received a copy of the GNU General Public License along
+   with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 #include <stdlib.h>
 #include <math.h>
@@ -47,6 +45,11 @@ $Id: pdftoepdf.cc,v 1.9 2006/09/01 18:06:52 hahe Exp hahe $
 
 #include "epdf.h"
 
+static const char _svn_version[] =
+    "$Id: pdftoepdf.cc 1224 2008-05-02 15:26:27Z oneiros $ $URL: http://scm.foundry.supelec.fr/svn/luatex/tags/beta-0.28.0/src/texk/web2c/luatexdir/image/pdftoepdf.cc $";
+
+#define one_hundred_bp  6578176 /* 7227 * 65536 / 72 */
+
 // This file is mostly C and not very much C++; it's just used to interface
 // the functions of xpdf, which happens to be written in C++.
 
@@ -74,10 +77,12 @@ $Id: pdftoepdf.cc,v 1.9 2006/09/01 18:06:52 hahe Exp hahe $
 // and &obj to get a pointer to the object.
 // It is no longer necessary to call Object::free explicitely.
 
+/* *INDENT-OFF* */
 class PdfObject {
   public:
     PdfObject() {               // nothing
-    } ~PdfObject() {
+    }
+    ~PdfObject() {
         iObject.free();
     }
     Object *operator->() {
@@ -92,6 +97,7 @@ class PdfObject {
   public:
     Object iObject;
 };
+/* *INDENT-ON* */
 
 // When copying the Resources of the selected page, all objects are copied
 // recusively top-down. Indirect objects however are not fetched during
@@ -99,20 +105,16 @@ class PdfObject {
 // appended into a linked list. Duplicates are checked and removed from the
 // list of indirect objects during appending.
 
-enum InObjType {
-    objFont,
-    objFontDesc,
-    objOther
-};
+enum InObjType { objFont, objFontDesc, objOther };
 
 struct InObj {
     Ref ref;                    // ref in original PDF
     InObjType type;             // object type
-    InObj *next;                // next entry in list of indirect objects
     integer num;                // new object number in output PDF
     fd_entry *fd;               // pointer to /FontDescriptor object structure
     integer enc_objnum;         // Encoding for objFont
     int written;                // has it been written to output PDF?
+    InObj *next;                // next entry in list of indirect objects
 };
 
 struct UsedEncoding {
@@ -121,89 +123,104 @@ struct UsedEncoding {
     UsedEncoding *next;
 };
 
-static InObj *inObjList;
+static XRef *xref = NULL;
+static InObj *inObjList = NULL;
 static UsedEncoding *encodingList;
 static GBool isInit = gFalse;
 
-// --------------------------------------------------------------------
-// Maintain list of open embedded PDF files
-// --------------------------------------------------------------------
+//**********************************************************************
+// Maintain AVL tree of open embedded PDF files
+
+static avl_table *PdfDocumentTree = NULL;
 
 struct PdfDocument {
-    char *file_name;
+    char *file_path;            // full file name including path
     PDFDoc *doc;
     XRef *xref;
     InObj *inObjList;
-    int occurences;             // number of references to the document; the doc can be
-    // deleted when this is negative
-    PdfDocument *next;
+    int occurences;             // number of references to the PdfDocument; it can be deleted when occurences == 0
 };
 
-static PdfDocument *pdfDocuments = 0;
+/* AVL sort PdfDocument into PdfDocumentTree by file_path */
 
-static XRef *xref = 0;
-
-// Returns pointer to PdfDocument record for PDF file.
-// Creates a new record if it doesn't exist yet.
-// xref is made current for the document.
-
-static PdfDocument *find_add_document(char *file_name)
+static int CompPdfDocument(const void *pa, const void *pb, void *p)
 {
-    PdfDocument *p = pdfDocuments;
-    while (p && strcmp(p->file_name, file_name) != 0)
-        p = p->next;
-    if (p) {
-        xref = p->xref;
-        (p->occurences)++;
-#ifdef DEBUG
-        fprintf(stderr, "\npdfTeX Debug: Incrementing %s (%d)\n", p->file_name,
-                p->occurences);
-#endif
-        return p;
-    }
-    p = new PdfDocument;
-    p->file_name = xstrdup(file_name);
-    p->xref = xref = 0;
-    p->occurences = 0;
-#ifdef DEBUG
-    fprintf(stderr, "\npdfTeX Debug: Creating %s (%d)\n", p->file_name,
-            p->occurences);
-#endif
-    GString *docName = new GString(p->file_name);
-    p->doc = new PDFDoc(docName);       // takes ownership of docName
-    if (!p->doc->isOk() || !p->doc->okToPrint()) {
-        pdftex_fail("xpdf: reading PDF image failed");
-    }
-    p->inObjList = 0;
-    p->next = pdfDocuments;
-    pdfDocuments = p;
-    return p;
+    return strcmp(((const PdfDocument *) pa)->file_path,
+                  ((const PdfDocument *) pb)->file_path);
 }
 
-// Deallocate a PdfDocument with all its resources
+// Returns pointer to PdfDocument structure for PDF file.
 
-static void delete_document(PdfDocument * pdf_doc)
+static PdfDocument *findPdfDocument(char *file_path)
 {
-    PdfDocument **p = &pdfDocuments;
-    while (*p && *p != pdf_doc)
-        p = &((*p)->next);
-    // should not happen:
-    if (!*p)
-        return;
-    // unlink from list
-    *p = pdf_doc->next;
-    // free pdf_doc's resources
-    InObj *r, *n;
-    for (r = pdf_doc->inObjList; r != 0; r = n) {
-        n = r->next;
-        delete r;
-    }
-    xref = pdf_doc->xref;
-    delete pdf_doc->doc;
-    xfree(pdf_doc->file_name);
-    delete pdf_doc;
+    PdfDocument *pdf_doc, tmp;
+    assert(file_path != NULL);
+    if (PdfDocumentTree == NULL)
+        PdfDocumentTree = avl_create(CompPdfDocument, NULL, &avl_xallocator);
+    assert(PdfDocumentTree != NULL);
+    tmp.file_path = file_path;
+    pdf_doc = (PdfDocument *) avl_find(PdfDocumentTree, &tmp);
+    return pdf_doc;
 }
 
+// Returns pointer to PdfDocument structure for PDF file.
+// Creates a new structure if it doesn't exist yet.
+
+static PdfDocument *refPdfDocument(char *file_path)
+{
+    PdfDocument *pdf_doc = findPdfDocument(file_path);
+    if (pdf_doc == NULL) {
+        pdf_doc = new PdfDocument;
+        pdf_doc->file_path = xstrdup(file_path);
+        void **aa = avl_probe(PdfDocumentTree, pdf_doc);
+        assert(aa != NULL);
+        GString *docName = new GString(pdf_doc->file_path);
+        pdf_doc->doc = new PDFDoc(docName);     // takes ownership of docName
+        if (!pdf_doc->doc->isOk() || !pdf_doc->doc->okToPrint())
+            pdftex_fail("xpdf: reading PDF image failed");
+        pdf_doc->xref = NULL;
+        pdf_doc->inObjList = NULL;
+        pdf_doc->occurences = 0;        // 0 = unreferenced
+#ifdef DEBUG
+        fprintf(stderr, "\nluatex Debug: Creating %s (%d)\n",
+                pdf_doc->file_path, pdf_doc->occurences);
+#endif
+    }
+    pdf_doc->occurences++;
+#ifdef DEBUG
+    fprintf(stderr, "\nluatex Debug: Incrementing %s (%d)\n",
+            pdf_doc->file_path, pdf_doc->occurences);
+#endif
+    return pdf_doc;
+}
+
+static void deletePdfDocument(PdfDocument *);
+
+// Called when an image has been written and its resources in image_tab are
+// freed and it's not referenced anymore.
+
+void unrefPdfDocument(char *file_path)
+{
+    PdfDocument *pdf_doc = findPdfDocument(file_path);
+    assert(pdf_doc != NULL);
+    assert(pdf_doc->occurences > 0);    // aim for point landing
+    pdf_doc->occurences--;
+#ifdef DEBUG
+    fprintf(stderr, "\nluatex Debug: Decrementing %s (%d)\n",
+            pdf_doc->file_path, pdf_doc->occurences);
+#endif
+    if (pdf_doc->occurences == 0) {
+#ifdef DEBUG
+        fprintf(stderr, "\nluatex Debug: Deleting %s\n", pdf_doc->file_path);
+#endif
+        void *a = avl_delete(PdfDocumentTree, pdf_doc);
+        assert((PdfDocument *) a == pdf_doc);
+        deletePdfDocument(pdf_doc);
+    }
+}
+
+//**********************************************************************
+
 // Replacement for
 //      Object *initDict(Dict *dict1){ initObj(objDict); dict = dict1; return this; }
 
@@ -216,8 +233,6 @@ static void initDictFromDict(PdfObject & obj, Dict * dict)
     }
 }
 
-// --------------------------------------------------------------------
-
 static int addEncoding(GfxFont * gfont)
 {
     UsedEncoding *n;
@@ -239,7 +254,7 @@ static int addEncoding(GfxFont * gfont)
         addInObj(objFontDesc, ref, fd, 0)
 
 #define addOther(ref) \
-        addInObj(objOther, ref, 0, 0)
+        addInObj(objOther, ref, NULL, 0)
 
 static int addInObj(InObjType type, Ref ref, fd_entry * fd, integer e)
 {
@@ -248,14 +263,14 @@ static int addInObj(InObjType type, Ref ref, fd_entry * fd, integer e)
         pdftex_fail("PDF inclusion: invalid reference");
     n->ref = ref;
     n->type = type;
-    n->next = 0;
+    n->next = NULL;
     n->fd = fd;
     n->enc_objnum = e;
     n->written = 0;
-    if (inObjList == 0)
+    if (inObjList == NULL)
         inObjList = n;
     else {
-        for (p = inObjList; p != 0; p = p->next) {
+        for (p = inObjList; p != NULL; p = p->next) {
             if (p->ref.num == ref.num && p->ref.gen == ref.gen) {
                 delete n;
                 return p->num;
@@ -264,7 +279,7 @@ static int addInObj(InObjType type, Ref ref, fd_entry * fd, integer e)
         }
         // it is important to add new objects at the end of the list,
         // because new objects are being added while the list is being
-        // written out.
+        // written out by writeRefs().
         q->next = n;
     }
     if (type == objFontDesc)
@@ -274,8 +289,6 @@ static int addInObj(InObjType type, Ref ref, fd_entry * fd, integer e)
     return n->num;
 }
 
-static void copyObject(Object *);
-
 static void copyName(char *s)
 {
     pdf_puts("/");
@@ -288,6 +301,8 @@ static void copyName(char *s)
     }
 }
 
+static void copyObject(Object *);
+
 static void copyDictEntry(Object * obj, int i)
 {
     PdfObject obj1;
@@ -381,8 +396,7 @@ static void copyFont(char *tag, Object * fontRef)
     }
     // Only handle included Type1 (and Type1C) fonts; anything else will be copied.
     // Type1C fonts are replaced by Type1 fonts, if REPLACE_TYPE1C is true.
-    if (fixed_replace_font
-		&& fontRef->fetch(xref, &fontdict)->isDict()
+    if (fixed_replace_font && fontRef->fetch(xref, &fontdict)->isDict()
         && fontdict->dictLookup("Subtype", &subtype)->isName()
         && !strcmp(subtype->getName(), "Type1")
         && fontdict->dictLookup("BaseFont", &basefont)->isName()
@@ -501,7 +515,7 @@ static char *convertNumToPDF(double n)
                 fval /= 10;
             }
         } else
-            buf[bindex++] = 0;
+            buf[bindex++] = '\0';
     }
     return (char *) buf;
 }
@@ -590,14 +604,14 @@ static void copyObject(Object * obj)
 static void writeRefs()
 {
     InObj *r;
-    for (r = inObjList; r != 0; r = r->next) {
+    for (r = inObjList; r != NULL; r = r->next) {
         if (!r->written) {
             Object obj1;
             r->written = 1;
             xref->fetch(r->ref.num, r->ref.gen, &obj1);
             if (r->type == objFont) {
                 assert(!obj1.isStream());
-                zpdf_begin_obj(r->num, 2);        // \pdfobjcompresslevel = 2 is for this
+                zpdf_begin_obj(r->num, 2);      // \pdfobjcompresslevel = 2 is for this
                 copyFontDict(&obj1, r);
                 pdf_puts("\n");
                 pdf_end_obj();
@@ -605,7 +619,7 @@ static void writeRefs()
                 if (obj1.isStream())
                     zpdf_begin_obj(r->num, 0);
                 else
-                    zpdf_begin_obj(r->num, 2);    // \pdfobjcompresslevel = 2 is for this
+                    zpdf_begin_obj(r->num, 2);  // \pdfobjcompresslevel = 2 is for this
                 copyObject(&obj1);
                 pdf_puts("\n");
                 pdf_end_obj();
@@ -620,20 +634,20 @@ static void writeEncodings()
     UsedEncoding *r, *n;
     char *glyphNames[256], *s;
     int i;
-    for (r = encodingList; r != 0; r = r->next) {
+    for (r = encodingList; r != NULL; r = r->next) {
         for (i = 0; i < 256; i++) {
             if (r->font->isCIDFont()) {
                 pdftex_warn
                     ("PDF inclusion: CID font included, encoding maybe wrong");
             }
-            if ((s = ((Gfx8BitFont *) r->font)->getCharName(i)) != 0)
+            if ((s = ((Gfx8BitFont *) r->font)->getCharName(i)) != NULL)
                 glyphNames[i] = s;
             else
                 glyphNames[i] = notdef;
         }
         epdf_write_enc(glyphNames, r->enc_objnum);
     }
-    for (r = encodingList; r != 0; r = n) {
+    for (r = encodingList; r != NULL; r = n) {
         n = r->next;
         delete r->font;
         delete r;
@@ -641,6 +655,7 @@ static void writeEncodings()
 }
 
 // get the pagebox according to the pagebox_spec
+
 static PDFRectangle *get_pagebox(Page * page, integer pagebox_spec)
 {
     if (pagebox_spec == pdf_box_spec_media)
@@ -659,40 +674,38 @@ static PDFRectangle *get_pagebox(Page * page, integer pagebox_spec)
     return page->getMediaBox(); // to make the compiler happy
 }
 
-
 // Reads various information about the PDF and sets it up for later inclusion.
 // This will fail if the PDF version of the PDF is higher than
 // minor_pdf_version_wanted or page_name is given and can not be found.
 // It makes no sense to give page_name _and_ page_num.
 // Returns the page number.
 
-integer
-read_pdf_info(char *image_name, char *page_name, integer page_num,
-              integer pagebox_spec, integer minor_pdf_version_wanted,
+void
+read_pdf_info(image_dict * idict, integer minor_pdf_version_wanted,
               integer pdf_inclusion_errorlevel)
 {
     PdfDocument *pdf_doc;
     Page *page;
     int rotate;
     PDFRectangle *pagebox;
-    float pdf_version_found, pdf_version_wanted;
+    float pdf_version_found, pdf_version_wanted, xsize, ysize, xorig, yorig;
+    assert(idict != NULL);
+    assert(img_type(idict) == IMG_TYPE_PDF);
     // initialize
-    if (!isInit) {
+    if (isInit == gFalse) {
         globalParams = new GlobalParams();
         globalParams->setErrQuiet(gFalse);
         isInit = gTrue;
     }
     // open PDF file
-    pdf_doc = find_add_document(image_name);
-    epdf_doc = (void *) pdf_doc;
-
+    pdf_doc = refPdfDocument(img_filepath(idict));
     // check PDF version
     // this works only for PDF 1.x -- but since any versions of PDF newer
     // than 1.x will not be backwards compatible to PDF 1.x, pdfTeX will
     // then have to changed drastically anyway.
     pdf_version_found = pdf_doc->doc->getPDFVersion();
     pdf_version_wanted = 1 + (minor_pdf_version_wanted * 0.1);
-    if (pdf_version_found > pdf_version_wanted) {
+    if (pdf_version_found > pdf_version_wanted + 0.01) {
         char msg[] =
             "PDF inclusion: found PDF version <%.1f>, but at most version <%.1f> allowed";
         if (pdf_inclusion_errorlevel > 0) {
@@ -701,77 +714,82 @@ read_pdf_info(char *image_name, char *page_name, integer page_num,
             pdftex_warn(msg, pdf_version_found, pdf_version_wanted);
         }
     }
-    epdf_num_pages = pdf_doc->doc->getCatalog()->getNumPages();
-    if (page_name) {
+    img_totalpages(idict) = pdf_doc->doc->getCatalog()->getNumPages();
+    if (img_pagename(idict)) {
         // get page by name
-        GString name(page_name);
+        GString name(img_pagename(idict));
         LinkDest *link = pdf_doc->doc->findDest(&name);
-        if (link == 0 || !link->isOk())
-            pdftex_fail("PDF inclusion: invalid destination <%s>", page_name);
+        if (link == NULL || !link->isOk())
+            pdftex_fail("PDF inclusion: invalid destination <%s>",
+                        img_pagename(idict));
         Ref ref = link->getPageRef();
-        page_num = pdf_doc->doc->getCatalog()->findPage(ref.num, ref.gen);
-        if (page_num == 0)
+        img_pagenum(idict) =
+            pdf_doc->doc->getCatalog()->findPage(ref.num, ref.gen);
+        if (img_pagenum(idict) == 0)
             pdftex_fail("PDF inclusion: destination is not a page <%s>",
-                        page_name);
+                        img_pagename(idict));
         delete link;
     } else {
         // get page by number
-        if (page_num <= 0 || page_num > epdf_num_pages)
-            pdftex_fail("PDF inclusion: required page does not exist <%i>",
-                        (int) epdf_num_pages);
-    }
-    // get the required page
-    page = pdf_doc->doc->getCatalog()->getPage(page_num);
-
-    // get the pagebox (media, crop...) to use.
-    pagebox = get_pagebox(page, pagebox_spec);
-    if (pagebox->x2 > pagebox->x1) {
-        epdf_orig_x_i = pagebox->x1;
-        epdf_width = pagebox->x2 - pagebox->x1;
-    } else {
-        epdf_orig_x_i = pagebox->x2;
-        epdf_width = pagebox->x1 - pagebox->x2;
-    }
-    if (pagebox->y2 > pagebox->y1) {
-        epdf_orig_y_i = pagebox->y1;
-        epdf_height = pagebox->y2 - pagebox->y1;
-    } else {
-        epdf_orig_y_i = pagebox->y2;
-        epdf_height = pagebox->y1 - pagebox->y2;
-    }
-
-    rotate = page->getRotate();
-    // handle page rotation and adjust dimens as needed
-    if (rotate != 0) {
-        if (rotate % 90 == 0) {
-            // handle only the simple case: multiple of 90s.
-            // these are the only values allowed according to the
-            // reference (v1.3, p. 78).
-            // 180 needs no special treatment here
-            register float f;
-            switch (rotate) {
-            case 90:
-                f = epdf_height;
-                epdf_height = epdf_width;
-                epdf_width = f;
-                break;
-            case 270:
-                f = epdf_height;
-                epdf_height = epdf_width;
-                epdf_width = f;
-                break;
-            }
-        }
+        if (img_pagenum(idict) <= 0
+            || img_pagenum(idict) > img_totalpages(idict))
+            pdftex_fail("PDF inclusion: required page <%i> does not exist",
+                        (int) img_pagenum(idict));
     }
     pdf_doc->xref = pdf_doc->doc->getXRef();
-    return page_num;
+    // get the required page
+    page = pdf_doc->doc->getCatalog()->getPage(img_pagenum(idict));
+
+    // get the pagebox (media, crop...) to use.
+    pagebox = get_pagebox(page, img_pagebox(idict));
+    if (pagebox->x2 > pagebox->x1) {
+        xorig = pagebox->x1;
+        xsize = pagebox->x2 - pagebox->x1;
+    } else {
+        xorig = pagebox->x2;
+        xsize = pagebox->x1 - pagebox->x2;
+    }
+    if (pagebox->y2 > pagebox->y1) {
+        yorig = pagebox->y1;
+        ysize = pagebox->y2 - pagebox->y1;
+    } else {
+        yorig = pagebox->y2;
+        ysize = pagebox->y1 - pagebox->y2;
+    }
+    // The following 4 parameters are raw. Do _not_ modify by /Rotate!
+    img_xsize(idict) = bp2int(xsize);
+    img_ysize(idict) = bp2int(ysize);
+    img_xorig(idict) = bp2int(xorig);
+    img_yorig(idict) = bp2int(yorig);
+
+    // Handle /Rotate parameter. Only multiples of 90 deg. are allowed
+    // (PDF Ref. v1.3, p. 78).
+    rotate = page->getRotate();
+    switch (((rotate % 360) + 360) % 360) {     // handles also neg. angles
+    case 0:
+        img_rotation(idict) = 0;
+        break;
+    case 90:
+        img_rotation(idict) = 3;        // PDF counts clockwise!
+        break;
+    case 180:
+        img_rotation(idict) = 2;
+        break;
+    case 270:
+        img_rotation(idict) = 1;
+        break;
+    default:
+        pdftex_warn
+            ("PDF inclusion: "
+             "/Rotate parameter in PDF file not multiple of 90 degrees.");
+    }
 }
 
-// writes the current epf_doc.
+// Writes the current epf_doc.
 // Here the included PDF is copied, so most errors that can happen during PDF
 // inclusion will arise here.
 
-void write_epdf(void)
+static void write_epdf1(image_dict * idict)
 {
     Page *page;
     PdfObject contents, obj1, obj2;
@@ -780,84 +798,48 @@ void write_epdf(void)
     char *key;
     char s[256];
     int i, l;
-    int rotate;
-    double scale[6] = { 0, 0, 0, 0, 0, 0 };
-    bool writematrix = false;
-    PdfDocument *pdf_doc = (PdfDocument *) epdf_doc;
-    (pdf_doc->occurences)--;
-#ifdef DEBUG
-    fprintf(stderr, "\npdfTeX Debug: Decrementing %s (%d)\n",
-            pdf_doc->file_name, pdf_doc->occurences);
-#endif
+    PdfDocument *pdf_doc = (PdfDocument *) findPdfDocument(img_filepath(idict));
+    assert(pdf_doc != NULL);
     xref = pdf_doc->xref;
     inObjList = pdf_doc->inObjList;
-    encodingList = 0;
-    page = pdf_doc->doc->getCatalog()->getPage(epdf_selected_page);
-    rotate = page->getRotate();
+    encodingList = NULL;
+    page = pdf_doc->doc->getCatalog()->getPage(img_pagenum(idict));
     PDFRectangle *pagebox;
+    float bbox[4];
     // write the Page header
-    pdf_puts("/Type /XObject\n");
-    pdf_puts("/Subtype /Form\n");
+    pdf_puts("/Type /XObject\n/Subtype /Form\n");
+    if (img_attr(idict) != NULL && strlen(img_attr(idict)) > 0)
+        pdf_printf("%s\n", img_attr(idict));
     pdf_puts("/FormType 1\n");
 
     // write additional information
     pdf_printf("/%s.FileName (%s)\n", pdfkeyprefix,
-               convertStringToPDFString(pdf_doc->file_name,
-                                        strlen(pdf_doc->file_name)));
-    pdf_printf("/%s.PageNumber %i\n", pdfkeyprefix, (int) epdf_selected_page);
+               convertStringToPDFString(pdf_doc->file_path,
+                                        strlen(pdf_doc->file_path)));
+    pdf_printf("/%s.PageNumber %i\n", pdfkeyprefix, (int) img_pagenum(idict));
     pdf_doc->doc->getDocInfoNF(&info);
     if (info.isRef()) {
         // the info dict must be indirect (PDF Ref p. 61)
         pdf_printf("/%s.InfoDict ", pdfkeyprefix);
         pdf_printf("%d 0 R\n", addOther(info.getRef()));
     }
-    // get the pagebox (media, crop...) to use.
-    pagebox = get_pagebox(page, epdf_page_box);
-
-    // handle page rotation
-    if (rotate != 0) {
-        if (rotate % 90 == 0) {
-            // this handles only the simple case: multiple of 90s but these
-            // are the only values allowed according to the reference
-            // (v1.3, p. 78).
-            // the image is rotated around its center.
-            // the /Rotate key is clockwise while the matrix is
-            // counterclockwise :-%
-            tex_printf(", page is rotated %d degrees", rotate);
-            switch (rotate) {
-            case 90:
-                scale[1] = -1;
-                scale[2] = 1;
-                scale[4] = pagebox->x1 - pagebox->y1;
-                scale[5] = pagebox->y1 + pagebox->x2;
-                writematrix = true;
-                break;
-            case 180:
-                scale[0] = scale[3] = -1;
-                scale[4] = pagebox->x1 + pagebox->x2;
-                scale[5] = pagebox->y1 + pagebox->y2;
-                writematrix = true;
-                break;          // width and height are exchanged
-            case 270:
-                scale[1] = 1;
-                scale[2] = -1;
-                scale[4] = pagebox->x1 + pagebox->y2;
-                scale[5] = pagebox->y1 - pagebox->x1;
-                writematrix = true;
-                break;
-            }
-            if (writematrix) {  // The matrix is only written if the image is rotated.
-                sprintf(s, "/Matrix [%.8f %.8f %.8f %.8f %.8f %.8f]\n",
-                        scale[0],
-                        scale[1], scale[2], scale[3], scale[4], scale[5]);
-                pdf_printf(stripzeros(s));
-            }
-        }
+    if (img_is_bbox(idict)) {
+        bbox[0] = int2bp(img_bbox(idict)[0]);
+        bbox[1] = int2bp(img_bbox(idict)[1]);
+        bbox[2] = int2bp(img_bbox(idict)[2]);
+        bbox[3] = int2bp(img_bbox(idict)[3]);
+    } else {
+        // get the pagebox (media, crop...) to use.
+        pagebox = get_pagebox(page, img_pagebox(idict));
+        bbox[0] = pagebox->x1;
+        bbox[1] = pagebox->y1;
+        bbox[2] = pagebox->x2;
+        bbox[3] = pagebox->y2;
     }
-
-    sprintf(s, "/BBox [%.8f %.8f %.8f %.8f]\n",
-            pagebox->x1, pagebox->y1, pagebox->x2, pagebox->y2);
+    sprintf(s, "/BBox [%.8f %.8f %.8f %.8f]\n", bbox[0], bbox[1], bbox[2],
+            bbox[3]);
     pdf_printf(stripzeros(s));
+    // The /Matrix calculation is replaced by transforms in out_img().
 
     // write the page Group if it's there
     if (page->getGroup() != NULL) {
@@ -873,12 +855,12 @@ void write_epdf(void)
 #  else
         // FIXME: currently we don't know how to handle Page Groups so we abort gracefully :-(
         pdftex_fail
-            ("PDF inclusion: Page Group detected which pdfTeX can't handle. Sorry.");
+            ("PDF inclusion: Page Group detected which luatex can't handle. Sorry.");
 #  endif
 #else
         // FIXME: currently we don't know how to handle Page Groups so we at least give a warning :-(
         pdftex_warn
-            ("PDF inclusion: Page Group detected which pdfTeX can't handle. Ignoring it.");
+            ("PDF inclusion: Page Group detected which luatex can't handle. Ignoring it.");
 #endif
     }
     // write the page Metadata if it's there
@@ -960,24 +942,39 @@ void write_epdf(void)
     writeRefs();
     // write out all used encodings (and delete list)
     writeEncodings();
-    // save object list, xref
+    // save object list
     pdf_doc->inObjList = inObjList;
-    pdf_doc->xref = xref;
+    assert(xref == pdf_doc->xref);      // xref should be unchanged
 }
 
-// Called when an image has been written and it's resources in image_tab are
-// freed and it's not referenced anymore.
-
-void epdf_delete()
+void write_epdf(image_dict * idict)
 {
-    PdfDocument *pdf_doc = (PdfDocument *) epdf_doc;
-    xref = pdf_doc->xref;
-    if (pdf_doc->occurences < 0) {
-#ifdef DEBUG
-        fprintf(stderr, "\npdfTeX Debug: Deleting %s\n", pdf_doc->file_name);
-#endif
-        delete_document(pdf_doc);
+    assert(idict != NULL);
+    write_epdf1(idict);
+    unrefPdfDocument(img_filepath(idict));
+}
+
+//**********************************************************************
+// Deallocate a PdfDocument with all its resources
+
+static void deletePdfDocument(PdfDocument * pdf_doc)
+{
+    assert(pdf_doc != NULL);
+    // free PdfDocument's resources
+    InObj *r, *n;
+    for (r = pdf_doc->inObjList; r != NULL; r = n) {
+        n = r->next;
+        delete r;
     }
+    delete pdf_doc->doc;
+    xfree(pdf_doc->file_path);
+    delete pdf_doc;
+}
+
+static void destroyPdfDocument(void *pa, void *pb)
+{
+    PdfDocument *pdf_doc = (PdfDocument *) pa;
+    deletePdfDocument(pdf_doc);
 }
 
 // Called when PDF embedding system is finalized.
@@ -985,13 +982,10 @@ void epdf_delete()
 
 void epdf_check_mem()
 {
-    if (isInit) {
-        PdfDocument *p, *n;
-        for (p = pdfDocuments; p; p = n) {
-            n = p->next;
-            delete_document(p);
-        }
-        // see above for globalParams
+    if (PdfDocumentTree != NULL)
+        avl_destroy(PdfDocumentTree, destroyPdfDocument);
+    PdfDocumentTree = NULL;
+    if (isInit == gTrue)
         delete globalParams;
-    }
+    isInit = gFalse;
 }
