@@ -283,7 +283,15 @@ static void hyppat_insert(
   i = hnj_string_hash (key) % HASH_SIZE;
   for (e = hashtab->entries[i]; e; e=e->next) {
     if (strcmp((char*)e->key,(char*)key)==0) {
-      if (e->u.hyppat) hnj_free(e->u.hyppat);
+      if (e->u.hyppat) {
+        if (hyppat && strcmp((char*)e->u.hyppat,(char*)hyppat)!=0) {
+          int s = maketexstring("Conflicting pattern ignored");
+          do_print_err(s);
+          error();
+          flush_str(s);
+        }
+        hnj_free(e->u.hyppat);
+      }
       e->u.hyppat = hyppat;
       hnj_free(key);
       return;
@@ -370,8 +378,12 @@ static void hnj_add_trans(
   int uni_ch
 ) {
   int num_trans;
-
-  if (uni_ch<32 || (uni_ch>126 && uni_ch<160)) {
+  /* TH: this test was a bit too strict, it is quite normal for old
+     patterns to have chars in the range 0-31 or 127-159 (inclusive). 
+     To ease the transition, let's only disallow NUL for now 
+     (this is probably a requirement of the code anyway).
+  */
+  if (uni_ch==0) {
     fprintf(stderr,"Character out of bounds: u%04x \n",uni_ch);
     exit(1);
   }
@@ -414,9 +426,9 @@ static const unsigned char* next_pattern(
   size_t* length,
   const unsigned char **buf
 ) {
-  const unsigned char *rover = *buf;
+  const unsigned char *here, *rover = *buf;
   while (*rover && isspace(*rover)) rover++;
-  const unsigned char *here = rover;
+  here = rover;
   while (*rover) {
     if (isspace(*rover)) {
       *length = rover-here;
@@ -433,8 +445,8 @@ static const unsigned char* next_pattern(
 static void init_hash(
   HashTab**h
 ) {
-  if (*h) return;
   int i;
+  if (*h) return;
   *h = hnj_malloc(sizeof(HashTab));
   for (i = 0; i < HASH_SIZE; i++) (*h)->entries[i] = NULL;
 }
@@ -443,8 +455,8 @@ static void init_hash(
 static void clear_state_hash(
   HashTab**h
 ) {
-  if (*h==NULL) return;
   int i;
+  if (*h==NULL) return;
   for (i = 0; i < HASH_SIZE; i++) {
     HashEntry *e, *next;
     for (e = (*h)->entries[i]; e; e = next) {
@@ -461,8 +473,8 @@ static void clear_state_hash(
 static void clear_hyppat_hash(
   HashTab**h
 ) {
-  if (*h==NULL) return;
   int i;
+  if (*h==NULL) return;
   for (i = 0; i < HASH_SIZE; i++) {
     HashEntry *e, *next;
     for (e = (*h)->entries[i]; e; e = next) {
@@ -594,6 +606,8 @@ void hnj_hyphen_load(
 
   const unsigned char* format;
   const unsigned char* begin = f;
+  unsigned char *pat;
+  char *org;
   while((format = next_pattern(&l,&f))!=NULL) {
     int i,j,e;
     /*
@@ -619,17 +633,17 @@ void hnj_hyphen_load(
       repl = hnj_strdup(repl + 1);
     }
     */
-    for (i=0,j=0,e=0; i<l; i++) {
+    for (i=0,j=0,e=0; (unsigned)i<l; i++) {
       if (format[i]>='0'&&format[i]<='9') j++;
       if (is_utf8_follow(format[i])) e++;
     }
     /* l-e   => number of _characters_ not _bytes_*/
     /* l-e-j => number of pattern characters*/
-    unsigned char *pat = (unsigned char*) malloc(1+l-j);
-             char *org = (         char*) malloc(2+l-e-j);
+    pat = (unsigned char*) malloc(1+l-j);
+    org = (         char*) malloc(2+l-e-j);
 			 /* remove hyphenation encoders (digits) from pat*/
     org[0] = '0';
-    for (i=0,j=0,e=0; i<l; i++) {
+    for (i=0,j=0,e=0; (unsigned)i<l; i++) {
       unsigned char c = format[i];
       if (is_utf8_follow(c)) {
         pat[j+e++] = c;
@@ -653,17 +667,18 @@ void hnj_hyphen_load(
     for (l=1; l<=wordsize; l++) {
       if (is_utf8_follow(word[l])) continue; /* Do not clip an utf8 sequence*/
       for (j=1; j<=l; j++) {
+        char *subpat_pat;
         int i = l-j;
         if (is_utf8_follow(word[i])) continue; /* Do not start halfway an utf8 sequence*/
-        char *subpat_pat;
         if ((subpat_pat = hyppat_lookup(dict->patterns,word+i,j))!=NULL) {
           char* newpat_pat;
           if ((newpat_pat = hyppat_lookup(dict->merged,word,l))==NULL) {
+            char *neworg;
             unsigned char *newword=(unsigned char*)malloc(l+1);
-            strncpy((char*)newword, (char*)word,l); newword[l]=0;
             int e=0;
+            strncpy((char*)newword, (char*)word,l); newword[l]=0;
             for (i=0; i<l; i++) if (is_utf8_follow(newword[i])) e++;
-            char *neworg = malloc(l+2-e);
+            neworg = malloc(l+2-e);
             sprintf(neworg,"%0*d",l+1-e,0); /* fill with right amount of '0'*/
             hyppat_insert(dict->merged,newword,combine(neworg,subpat_pat));
           } else {
@@ -693,10 +708,11 @@ void hnj_hyphen_load(
       last_state = state_num;
       ch = word[j];
       if (ch>=0x80) {
-        int i=1;
+        int m;
+		int i=1;
         while (is_utf8_follow(word[j-i])) i++;
         ch = word[j-i] & mask[i];
-        int m = j-i;
+        m = j-i;
         while (i--) {
           ch = (ch<<6)+(0x3F & word[j-i]);
         }
@@ -750,6 +766,9 @@ void hnj_hyphen_hyphenate(
   halfword right,
   lang_variables *lan
 ) {
+  int char_num;
+  halfword here;
+  int state = 0;
   /* +2 for dots at each end, +1 for points /outside/ characters*/
   int ext_word_len = length+2;
   int hyphen_len   = ext_word_len+1;
@@ -761,7 +780,6 @@ void hnj_hyphen_hyphenate(
   set_vlink(end_point,get_vlink(last));
   set_vlink(last,end_point);
 
-  int char_num;
   for (char_num = 0; char_num < hyphen_len; char_num++) {
 	/*  hyphens[char_num*2] = '0';    */ /* LATER */
 	/*  hyphens[char_num*2+1] = '0';    */ /* LATER */
@@ -771,11 +789,9 @@ void hnj_hyphen_hyphenate(
   hyphens[hyphen_len] = 0;
 
   /* now, run the finite state machine */
-  int state = 0;
-  halfword here;
   for (char_num=0, here=begin_point; here!=end_point; here=get_vlink(here)) {
 
-    int ch = get_character(here);
+    int ch = get_lc_code(get_character(here));
 
     while (state!=-1) {
 	  /*   printf("%*s%s%c",char_num-strlen(get_state_str(state)),"",get_state_str(state),(char)ch);*/
@@ -783,9 +799,10 @@ void hnj_hyphen_hyphenate(
       int k;
       for (k = 0; k < hstate->num_trans; k++) {
         if (hstate->trans[k].uni_ch == ch) {
-          state = hstate->trans[k].new_state;
+          char *match;
+		  state = hstate->trans[k].new_state;
 		  /*       printf(" state %d\n",state);*/
-          char *match = dict->states[state].match;
+          match = dict->states[state].match;
           if (match) {
             /* +2 because:
 			 *  1 string length is one bigger than offset
